@@ -48,7 +48,8 @@ from django.utils.translation import ugettext_lazy as _
 
 from quickapi.http import JSONResponse, MESSAGES
 from quickapi.conf import (settings, QUICKAPI_DEFINED_METHODS,
-    QUICKAPI_ONLY_AUTHORIZED_USERS, QUICKAPI_DEBUG, DEBUG, SITE_ID)
+    QUICKAPI_ONLY_AUTHORIZED_USERS, QUICKAPI_DEBUG, DEBUG, SITE_ID,
+    QUICKAPI_SWITCH_LANGUAGE, QUICKAPI_SWITCH_LANGUAGE_AUTO)
 
 import traceback, json as jsonlib, decimal
 
@@ -68,32 +69,72 @@ if 'django.contrib.sites' in settings.INSTALLED_APPS:
 else:
     site = None
 
-def switch_language(request):
+def switch_language(request, code=None):
+    """
+    Переключает язык для приложения, если такое переключение не
+    запрещено в настройках.
+    """
     old_language = translation.get_language()
+
+    if not QUICKAPI_SWITCH_LANGUAGE and not code:
+        # Disabled switching for quickapi only
+        return old_language, None
+
     new_language = None
-    if getattr(settings, 'LANGUAGES', None) and \
-    'django.middleware.locale.LocaleMiddleware' in settings.MIDDLEWARE_CLASSES:
+
+    if code:
+        new_language = code
+    elif 'language' in request.POST:
+        new_language = request.POST.get('language')
+    elif QUICKAPI_SWITCH_LANGUAGE_AUTO and hasattr(request, 'LANGUAGE_CODE'):
+        new_language = request.LANGUAGE_CODE
+
+    if new_language:
         try:
-            new_language = translation.activate(request.LANGUAGE_CODE)
+            new_language = translation.activate(new_language)
         except:
             translation.activate(old_language)
+            new_language = None
+
     return old_language, new_language
 
 @csrf_exempt
 def test(request):
-    """ Тестовый ответ """
+    """
+    Test response
+    """
+
+    default_language, request_language = switch_language(request)
+
     data = {
-        'integer': 9999,
-        'float': 9999.999,
-        'decimal': decimal.Decimal('9999.999'),
-        'boolean': True,
+        'REMOTE_ADDR': request.META.get("HTTP_X_REAL_IP", request.META.get("REMOTE_ADDR", None)),
+        'REMOTE_HOST': request.META.get("REMOTE_HOST", None),
+        'default language': default_language,
+        'request language': request_language,
         'string': _('String in your localization'),
         'datetime': timezone.now(),
-        'list': [9999, True, 'String'],
-        'dict': {'a': 1, 'b': 2, 'c': 3}
+        'is_authenticate': not request.user.is_anonymous(),
     }
     return JSONResponse(data=data)
 
+test.__doc__ = _(
+""" *Test response*
+
+    #### Request parameters
+    Nothing
+
+    #### Returned object
+    `{
+        'REMOTE_ADDR': '127.0.0.1' || null,
+        'REMOTE_HOST': 'example.org' || null,
+        'default language': 'en',
+        'request language': 'ru',
+        'string': 'String in your localization',
+        'datetime': '2013-01-01T00:00:00.000Z',
+        'is_authenticate': true,
+    }`
+""")
+
 def drop_space(doc):
     """ Удаление начальных и конечных пробелов в документации.
         Обратное слияние строк в текст зависит от наличия markdown
@@ -119,6 +160,19 @@ def drop_space(doc):
             L.append(s.strip(' '))     # или полностью очищенную строку
     return BIT.join(L)
 
+def get_doc(method):
+    text = drop_space(method.__doc__)
+    try:
+        text = markdown(text.decode('utf-8'))
+    except:
+        try:
+            text = markdown(text)
+        except:
+            pass
+    return text
+
+TEST_METHOD_DOC = lambda: get_doc(test)
+
 def get_methods(dic=QUICKAPI_DEFINED_METHODS):
     """ Преобразует словарь заданных строками методов, реальными
         объектами функций.
@@ -138,15 +192,11 @@ def get_methods(dic=QUICKAPI_DEFINED_METHODS):
                 print colorize(unicode(e), fg='red')
                 method = None
         if method:
-            text = drop_space(method.__doc__)
-            try:
-                text = markdown(text.decode('utf-8'))
-            except UnicodeDecodeError:
-                try:
-                    text = markdown(text)
-                except:
-                    pass
-            methods[key] = {'method': method, 'doc': text, 'name': key}
+            methods[key] = {
+                'method': method,
+                'doc': lambda: get_doc(method),
+                'name': key
+            }
 
     return methods
 
@@ -199,6 +249,8 @@ def index(request, dict_methods=None, methods=None):
     ctx = {}
     ctx['site'] = site
     ctx['methods'] = methods.values()
+    ctx['test_method_doc'] = TEST_METHOD_DOC if not 'quickapi.test' in methods else None
+
     return render_to_response('quickapi/index.html', ctx,
                             context_instance=RequestContext(request,))
 
@@ -266,7 +318,7 @@ def run(request, methods):
         user = authenticate(username=username, password=password)
         if user is not None and user.is_active:
             login(request, user)
-        elif QUICKAPI_ONLY_AUTHORIZED_USERS:
+        elif QUICKAPI_ONLY_AUTHORIZED_USERS and method != 'quickapi.test':
             return JSONResponse(status=401)
 
     if QUICKAPI_DEBUG:
@@ -275,15 +327,24 @@ def run(request, methods):
         p = '\tmethod\t\t\t== %s' % method
         print colorize(p, fg='blue')
 
-    try:
-        real_method = methods[method]['method']
-    except Exception as e:
-        msg = traceback.format_exc(e)
+    if method in methods:
+        try:
+            real_method = methods[method]['method']
+        except Exception as e:
+            msg = traceback.format_exc(e)
+            if DEBUG:
+                print msg
+            else:
+                msg = MESSAGES[405]
+            return JSONResponse(status=405, message=msg)
+    elif method == 'quickapi.test':
+        real_method = test
+    else:
+        msg = _('Method `%s` does not exist') % method
         if DEBUG:
             print msg
-        else:
-            msg = MESSAGES[405]
         return JSONResponse(status=405, message=msg)
+
     try:
         return real_method(request, **kwargs)
     except Exception as e:
